"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResponseWaiter = void 0;
class ResponseWaiter {
    constructor(page, options) {
        this.options = Object.assign({}, options);
        this.page = page;
        this.requestsCount = 0;
        if (this.options.resetOnNavigate == undefined) {
            this.options.resetOnNavigate = true;
        }
        if (!this.options.timeout) {
            this.options.timeout = 100;
        }
        this.requestHandler = (request) => {
            if (!this.shouldWait(request)) {
                return;
            }
            this.options.debug && console.log(`wait for request ${request.url()}`);
            ++this.requestsCount;
        };
        this.responseHandler = (response) => {
            if (!this.shouldWait(response.request())) {
                return;
            }
            this.requestsCount > 0 && --this.requestsCount;
            if (this.options.onResponse) {
                this.options.onResponse(response);
            }
        };
        this.framenavigatedHandler = () => {
            this.options.debug && console.log("resetting waiting");
            this.requestsCount = 0;
        };
        this.requestfailedHandler = (request) => {
            if (!this.shouldWait(request)) {
                return;
            }
            this.options.debug && console.log(`requests failed ${request.url()} ${request.failure().errorText}`);
            this.requestsCount > 0 && --this.requestsCount;
        };
    }
    shouldWait(request) {
        return this.options.waitFor && this.options.waitFor(request);
    }
    listen() {
        if (this.isListening) {
            return;
        }
        this.page.on('request', this.requestHandler);
        if (this.options.resetOnNavigate) {
            this.page.on('framenavigated', this.framenavigatedHandler);
        }
        this.page.on('requestfailed', this.requestfailedHandler);
        this.page.on('response', this.responseHandler);
        this.isListening = true;
    }
    stopListening() {
        if (!this.isListening) {
            return;
        }
        this.page.off('request', this.requestHandler);
        this.page.off('requestfailed', this.requestfailedHandler);
        this.page.off('response', this.responseHandler);
        this.page.off('framenavigated', this.framenavigatedHandler);
        this.isListening = false;
    }
    async wait() {
        if (!this.isListening) {
            return false;
        }
        this.options.debug && console.log('waiting...');
        do {
            this.options.debug && console.log(`waiting for ${this.requestsCount} pending reqs`);
            await this.page.waitForTimeout(this.options.timeout);
        } while (this.requestsCount > 0);
        return true;
    }
    getRequestsCount() {
        return this.requestsCount;
    }
}
exports.ResponseWaiter = ResponseWaiter;
exports.default = ResponseWaiter;
//# sourceMappingURL=index.js.map